#include <ArduinoBLE_P.h>
/*
 * Copied from an existing sketch the relevant parts and modified / added what is necessary for the test.
 * To show how often the function readValue() is called, in the file BLELocalCharateristic() readValue().
 *
 * Serial.print("read: ");
 * Serial.print(length);
 * Serial.print(" offset: ");
 * Serial.print(offset);
 * Serial.print(" AllWasReadBefore ");
 * Serial.println(AllWasReadBefore);
 *
 * The sending data is 33 bytes long , but will be sliding one character left with each send.
 *
 * I used 'nRF Connect' on my mobile as central.
 *
 * This sketch waits for connect event. Then it sends a string. On the 'nrf_connect' I perform a read (down_arrow) once.
 * It shows the string that was created by send_data before. With read (down_arrow) I see a new string.
 *
 * As you can see from below 22 bytes are read everytime (based on the MTU size agreed with nrf_Connect)
 *
Test. Compiled: 14:19:34

ready to go
Connected event, central: 48:cc:26:be:83:ad
written: ABCDEFGHIJKLMNOPQRSTUVW0123456789    // right after connect I write the FIRST value (this can be any dummy)
length: 33
CALLED!                                       // now I pressed read on the mobile and BLERead is called.
written: BCDEFGHIJKLMNOPQRSTUVW0123456789A    // the previous value is now overwritten and send with THIS read (starting with B)
length: 33
read: 22 offset: 0 AllWasReadBefore 0  
read: 11 offset: 22 AllWasReadBefore 1        // AllWasReadBefore is now set to indicate complete value was send
CALLED!                                       // now I pressed read again on the mobile and BLERead is called.
written: CDEFGHIJKLMNOPQRSTUVW0123456789AB    // the previous value is now overwritten and send with THIS read (starting with C)
length: 33
read: 22 offset: 0 AllWasReadBefore 0
read: 11 offset: 22 AllWasReadBefore 1        // AllWasReadBefore is now set to indicate complete value was send
CALLED!
written: DEFGHIJKLMNOPQRSTUVW0123456789ABC
length: 33
read: 22 offset: 0 AllWasReadBefore 0
read: 11 offset: 22 AllWasReadBefore 1

*/
/////////////////////////////////////////////////////////////////////////////////
// Performed modications ( 1 place)
/////////////////////////////////////////////////////////////////////////////////
/*

ArduinoBLE/src/local/BLELocalCharacteristic.cpp

CHANGE readValue() to :

void BLELocalCharacteristic::readValue(BLEDevice device, uint16_t offset, uint8_t value[], int length)
{
  // setting to TRUE at BEGINNING will force a call BLERead to enable overwrite the current value.  (set with a previous writeValue())
  // setting to FALSE at BEGINNING, it will first send the current value.  (set with a previous writeValue())

  // once the complete value has been send, with the following readValue() call it will
  // call BLERead to overwrite the current value. { Nov 22 paulvha}
  static bool AllWasReadBefore = true;

  // if the complete previous value was read before, allowed it to be updated
  if (AllWasReadBefore) {

     // allow call back (if set)
     if (_eventHandlers[BLERead]) {
          _eventHandlers[BLERead](device, BLECharacteristic(this));
     }

     AllWasReadBefore = false;
  }

  // copy the value
  memcpy(value, _value + offset, length);

  // check complete message has been read
  if (offset + length >= _valueLength) {
     AllWasReadBefore = true;
  }

 // Serial.print("read: ");
 // Serial.print(length);
 // Serial.print(" offset: ");
 // Serial.print(offset);
 // Serial.print(" AllWasReadBefore ");
 // Serial.println(AllWasReadBefore);
}

*/

const char BLE_PERIPHERAL_NAME[] = "Peripheral BLE";

// create the service
BLEService Service("19B10010-E8F2-537E-4F6C-D104768A1214");

// create characteristic and allow remote device to read and write
BLEStringCharacteristic rCharacteristic("19B10011-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite, 100);

bool Onetime = true;
bool Connected = false;
String Test="ABCDEFGHIJKLMNOPQRSTUVW0123456789";
int8_t t_offset = 0;

/*
 * called when all reading has been done
 */
void iter_temp_log(BLEDevice *central, BLECharacteristic c) {
    Serial.println("CALLED!");
    Send_data();
}

/*
 * send a sliding string
 */
void Send_data(){
  int i,j;
  String t_send;

  j=t_offset++;

  if (t_offset == Test.length()) t_offset=0;

  // for each letter
  for (i = 0; i < Test.length();i++){

   // copy from start offset
   t_send += Test[j++];

   // loop around
   if (j == Test.length()) j = 0;
  }

  Serial.print("written: ");
  Serial.println(t_send);
  Serial.print("length: ");
  Serial.println(t_send.length());

  rCharacteristic.writeValue(t_send);
}

void setup() {
   Serial.begin(115200);
   delay(1000);
   Serial.printf("Test. Compiled: %s\n\r", __TIME__);

  ///////////////////////////////////////////////////
  // BLE INIT
  ///////////////////////////////////////////////////
  // begin initialization
  if (!BLE.begin()) {
    Serial.println(F("starting BLE failed!\r"));
    while (1);
  }

  // set the local name peripheral advertises
  BLE.setLocalName(BLE_PERIPHERAL_NAME);

  // set the UUID for the service this peripheral advertises:
  BLE.setAdvertisedService(Service);

  // add the characteristics to the service
  Service.addCharacteristic(rCharacteristic);

  // add the service
  BLE.addService(Service);

  // set connection handlers
  BLE.setEventHandler(BLEConnected, blePeripheralConnectHandler);
  BLE.setEventHandler(BLEDisconnected, blePeripheralDisconnectHandler);

  // start advertising
  BLE.advertise();

  rCharacteristic.setEventHandler(BLERead, (BLECharacteristicEventHandler) iter_temp_log);

  Serial.println("ready to go");
}

void loop() {

  // wait for connection
  while (! Connected){
    BLE.poll();
    delay(1000);     // don't hammer
  }

  if (Onetime){      // send first time after connect
    delay(1000);     // wait for central to settle
    Send_data();
    Onetime = false;
  }

  BLE.poll();
  delay(500);        // don't hammer
}

/**
 * called when central connects
 */
void blePeripheralConnectHandler(BLEDevice central) {
  // central connected event handler
  Serial.print("Connected event, central: ");
  Serial.println(central.address());
  Connected = true;
  Onetime = true;
  t_offset=0;
}

/**
 * called when central disconnects
 */
void blePeripheralDisconnectHandler(BLEDevice central) {
  // central disconnected event handler
  Serial.print("Disconnected event, central: ");
  Serial.println(central.address());
  Connected = false;      // block loop and wait for new connection
}
